<?php
/**
 * TSV export for SMW Queries, forked from CSV export class in SMW
 */

class FE_tsv extends SMWResultPrinter {
	
	protected $m_sep;

	/**
	 * @see SMWResultPrinter::handleParameters
	 * 
	 * @since 1.7
	 * 
	 * @param array $params
	 * @param $outputmode
	 */
	protected function handleParameters( array $params, $outputmode ) {
		parent::handleParameters( $params, $outputmode );
		
		$this->m_sep = "\t";
	}

	public function getMimeType( $res ) {
		return 'text/plain';
	}

	public function getFileName( $res ) {
		return 'result.txt';
	}

	public function getQueryMode( $context ) {
		return ( $context == SMWQueryProcessor::SPECIAL_PAGE ) ? SMWQuery::MODE_INSTANCES : SMWQuery::MODE_NONE;
	}

	public function getName() {
		return wfMsg( 'sstar_smw_printername_tsv' );
	}

	protected function getResultText( SMWQueryResult $res, $outputmode ) {
		$result = '';
		
		if ( $outputmode == SMW_OUTPUT_FILE ) { // make TSV file
			$tsv = fopen( 'php://temp', 'r+' );
			
			if ( $this->mShowHeaders ) {
				$header_items = array();
				
				foreach ( $res->getPrintRequests() as $pr ) {
					$header_items[] = $pr->getLabel();
				}
				
				fwrite( $tsv, implode($header_items, $this->m_sep)."\n" );
			}
			
			while ( $row = $res->getNext() ) {
				$row_items = array();
				
				foreach ( $row as /* SMWResultArray */ $field ) {
					$growing = array();
					
					while ( ( $object = $field->getNextDataValue() ) !== false ) {
						$growing[] = Sanitizer::decodeCharReferences( $object->getWikiValue() );
					} 
					
					$row_items[] = implode( ',', $growing );
				}
				
				fwrite( $tsv, implode($row_items, $this->m_sep)."\n" );
			}

			rewind( $tsv );
			$result .= stream_get_contents( $tsv );
		} else { // just make link to feed
			if ( $this->getSearchLabel( $outputmode ) ) {
				$label = $this->getSearchLabel( $outputmode );
			} else {
				$label = wfMsgForContent( 'sstar_smw_tsv_link' );
			}

			$link = $res->getQueryLink( $label );
			$link->setParameter( 'tsv', 'format' );
			$link->setParameter( $this->m_sep, 'sep' );
			
			if ( array_key_exists( 'mainlabel', $this->params ) && $this->params['mainlabel'] !== false ) {
				$link->setParameter( $this->params['mainlabel'], 'mainlabel' );
			}
				
			$link->setParameter( $this->mShowHeaders ? 'show' : 'hide', 'headers' );
			
			if ( array_key_exists( 'limit', $this->params ) ) {
				$link->setParameter( $this->params['limit'], 'limit' );
			} else { // use a reasonable default limit
				$link->setParameter( 100, 'limit' );
			}
			
			$result .= $link->getText( $outputmode, $this->mLinker );
			$this->isHTML = ( $outputmode == SMW_OUTPUT_HTML ); // yes, our code can be viewed as HTML if requested, no more parsing needed
		}
		return $result;
	}

#	public function getParameters() {
#		$params = array_merge( parent::getParameters(), $this->exportFormatParameters() );
#		
#		$params['sep'] = new Parameter( 'sep' );
#		$params['sep']->setMessage( 'smw-paramdesc-csv-sep' );
#		$params['sep']->setDefault( ',' );
#		
#		return $params;
#	}

}
